import os

NT_AUTH = os.getenv("NT_AUTH", default="")
NT_TASKS_DB_ID = os.getenv("NT_TASKS_DB_ID", default="")
NT_STATS_DB_ID = os.getenv("NT_STATS_DB_ID", default="")
NT_EXPENSES_DB_ID = os.getenv("NT_EXPENSES_DB_ID", default="")
